VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "PScriptObject"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = True
Attribute VB_Exposed = False
Option Explicit

Public Enum POtherErrors
    poeNoError = 0
    poeShowedUsage = 1
    poeBadExtension = 2
    poeScriptNotFound = 3
    poeScriptError = 4
End Enum

Private Const INFINITE = &HFFFFFFFF
Private Const SYNCHRONIZE = &H100000
Private Const PROCESS_QUERY_INFORMATION = &H400&
  
Private Declare Function CloseHandle Lib "kernel32" ( _
    ByVal hObject As Long) As Long

Private Declare Function ExpandEnvironmentStringsAPI Lib "kernel32" _
    Alias "ExpandEnvironmentStringsW" ( _
    ByVal lpSrc As Long, _
    ByVal lpDst As Long, _
    ByVal nSize As Long) As Long

Private Declare Function GetExitCodeProcess Lib "kernel32" ( _
    ByVal hProcess As Long, _
    lpExitCode As Long) As Long

Private Declare Function OpenProcess Lib "kernel32" ( _
    ByVal dwDesiredAccess As Long, _
    ByVal bInheritHandle As Long, _
    ByVal dwProcessId As Long) As Long

Private Declare Sub SleepAPI Lib "kernel32" Alias "Sleep" (ByVal dwMilliseconds As Long)

Private Declare Function WaitForSingleObject Lib "kernel32" ( _
    ByVal hHandle As Long, _
    ByVal dwMilliseconds As Long) As Long

Private mArguments As Collection
Private mArgumentsLine As String
Private mLanguage As String
Private mOtherErrorInfo As String
Private mScriptFullName As String
Private mScriptPath As String
Private mScriptTimeout As Long

'--------------------------------------------------------------------------
'Public members

Public Property Get AppDataLocal() As String
    Const ssfLOCALAPPDATA = &H1C&
    
    AppDataLocal = CreateObject("Shell.Application").NameSpace(ssfLOCALAPPDATA).Self.Path
End Property

Public Property Get Arguments() As Collection
    Set Arguments = mArguments
End Property

Public Property Get ArgumentsLine() As String
    ArgumentsLine = mArgumentsLine
End Property

Public Property Get CD() As String
    CD = CurDir$()
End Property

Public Property Let CD(ByVal Path As String)
    ChDir Path
    If Mid$(Path, 2, 1) = ":" Then ChDrive Path
End Property

Public Sub CopyDir( _
    ByVal Source As String, _
    ByVal Dest As String)
    
    Dim SubDirs As Collection
    Dim DirItem As String
    Dim FullPath As String
    Dim SubDirInfo As Variant
    Dim Parts() As String
    Const ATTRS As Long = vbDirectory Or vbNormal Or vbReadOnly Or vbHidden Or vbArchive
    
    MkDir Dest
    Set SubDirs = New Collection
    DirItem = Dir$(Source & "\*.*", ATTRS)
    Do Until Len(DirItem) = 0
        FullPath = Source & "\" & DirItem
        If (GetAttr(FullPath) And vbDirectory) <> 0 Then
            If Not (DirItem = "." Or DirItem = "..") Then
                SubDirs.Add FullPath & vbNullChar & Dest & "\" & DirItem
            End If
        Else
            FileCopy FullPath, Dest & "\" & DirItem
        End If
        DirItem = Dir$()
    Loop
    For Each SubDirInfo In SubDirs
        Parts = Split(SubDirInfo, vbNullChar)
        CopyDir Parts(0), Parts(1)
    Next
End Sub

Public Sub CopyFile(ByVal Source As String, ByVal Dest As String)
    FileCopy Source, Dest
End Sub

Public Sub Del(ByVal File As String)
    'Wildcards allowed.
    Kill File
End Sub

Public Function Environment(ByVal Variable As Variant) As String
    'Can be variable name String or a number (Integer, Long).
    'Number out of range returns an empty String.
    Environment = Environ$(Variable)
End Function

Public Function Exists(ByVal FileOrDir As String) As Boolean
    On Error Resume Next
    GetAttr FileOrDir
    Exists = Err.Number = 0
End Function

Public Function ExpandEnvironmentStrings(ByVal EnvString As String) As String
    Dim Result As Long
    Dim Expanded As String
    
    Result = ExpandEnvironmentStringsAPI(StrPtr(EnvString), 0, 0)
    If Result = 0 Then
        GoTo ErrorExit
    Else
        Expanded = String$(Result - 1, 0)
        Result = ExpandEnvironmentStringsAPI(StrPtr(EnvString), StrPtr(Expanded), Result)
        If Result = 0 Then GoTo ErrorExit
    End If
    ExpandEnvironmentStrings = Expanded
    Exit Function
    
ErrorExit:
    Err.Raise &H80049600, TypeName(Me), "Error expanding environment strings: " & CStr(Err.LastDllError)
End Function

Public Sub MkDir(ByVal Path As String)
    VBA.MkDir Path
End Sub

Public Sub Quit()
    Err.Raise 51 'Internal error, catch this in Script Control error handler!
End Sub

Public Sub Rename(ByVal OldPath As String, ByVal NewPath As String)
    'Can rename a file or dir.  Will move a file across drives
    'but not a Dir.
    Name OldPath As NewPath
End Sub

Public Sub RmDir(ByVal Path As String, Optional ByVal Deep As Boolean = False)
    
    Dim SubDirs As Collection
    Dim DirItem As String
    Dim FullPath As String
    Dim SubDirInfo As Variant
    Const ATTRS As Long = vbDirectory Or vbNormal Or vbReadOnly Or vbHidden Or vbArchive
    
    If Deep Then
        Set SubDirs = New Collection
        DirItem = Dir$(Path & "\*.*", ATTRS)
        Do Until Len(DirItem) = 0
            FullPath = Path & "\" & DirItem
            If (GetAttr(FullPath) And vbDirectory) <> 0 Then
                If Not (DirItem = "." Or DirItem = "..") Then
                    SubDirs.Add FullPath
                End If
            Else
                Kill FullPath
            End If
            DirItem = Dir$()
        Loop
        For Each SubDirInfo In SubDirs
            RmDir SubDirInfo, Deep
        Next
    End If
    VBA.RmDir Path
End Sub

Public Function RndSubDirOf(ByVal Path As String) As String
    Dim NewDir As String
    
    Do
        NewDir = Path & "\X" & Hex$(CLng(&HFFFFFFF * Rnd()))
    Loop While Exists(NewDir)
    RndSubDirOf = NewDir
End Function

Public Function Run( _
    ByVal Program As String, _
    Optional ByVal ProgArgs As String, _
    Optional ByVal WindowStyle As VbAppWinStyle = vbNormalFocus, _
    Optional ByVal WaitOnReturn As Boolean = False) As Long

    Dim pidProcess As Long
    Dim hProcess As Long

    If vbHide > WindowStyle Or WindowStyle > vbMinimizedNoFocus Then
        Err.Raise &H80049602, TypeName(Me), "WindowStyle must be 0 to 6"
    Else
        On Error Resume Next
        pidProcess = Shell(Trim$("""" & Program & """ " & ProgArgs), WindowStyle)
        On Error GoTo 0
        If pidProcess = 0 Then
            Err.Raise &H80049604, TypeName(Me), "Failed to run the program"
        Else
            If WaitOnReturn Then
                hProcess = OpenProcess(SYNCHRONIZE _
                                    Or PROCESS_QUERY_INFORMATION, 0, pidProcess)
                If hProcess <> 0 Then
                    WaitForSingleObject hProcess, INFINITE
                    If GetExitCodeProcess(hProcess, Run) <> 0 Then
                        CloseHandle hProcess
                    Else
                        CloseHandle hProcess
                        Err.Raise &H80049606, _
                                  TypeName(Me), _
                                  "Failed to retrieve exit code, error " _
                                & CStr(Err.LastDllError)
                    End If
                Else
                    Err.Raise &H80049608, _
                              TypeName(Me), _
                              "Failed to open child process, error " _
                            & CStr(Err.LastDllError)
                End If
            End If
        End If
    End If
End Function

Public Property Get ScriptFullName() As String
    ScriptFullName = mScriptFullName
End Property

Public Property Get ScriptPath() As String
    ScriptPath = mScriptPath
End Property

Public Sub Sleep(ByVal Milliseconds As Long)
    SleepAPI Milliseconds
End Sub

'--------------------------------------------------------------------------
'Friend members

Friend Property Get Language() As String
    Language = mLanguage
End Property

Friend Property Get OtherErrorInfo() As String
    OtherErrorInfo = mOtherErrorInfo
End Property

Friend Property Get ScriptTimeout() As Long
    ScriptTimeout = mScriptTimeout
End Property

Friend Function ParseCommands() As POtherErrors
    Dim strCommand As String
    Dim Parts() As String
    Dim I As Integer
    Dim ScriptArgsBegin As Integer
    Dim ScriptArgsPos As Integer
    
    strCommand = Trim$(Command$())
    If Len(strCommand) < 1 Then
        Usage
        ParseCommands = poeShowedUsage
        Exit Function
    End If
    'Dequote command line.
    Parts = Split(strCommand, """")
    For I = 0 To UBound(Parts) Step 2
        Parts(I) = Replace$(Parts(I), " ", vbNullChar)
    Next
    'Split into Parts.
    Parts = Split(Join$(Parts, ""), vbNullChar)
    Select Case UCase$(Left$(Parts(0), 2))
        Case "?", "-?", "/?", "//"
            Usage
            ParseCommands = poeShowedUsage
        Case Else
            'Get script file full name.
            mScriptFullName = Dir$(Parts(0), vbNormal Or vbReadOnly Or vbHidden)
            If Len(mScriptFullName) < 1 Then
                ParseCommands = poeScriptNotFound
                Exit Function
            End If
            Select Case Mid$(mScriptFullName, 2, 1)
                Case ":", "\"
                    'We should have a full path.
                Case Else
                    'Relative path.
                    mScriptFullName = CurDir$() & "\" & mScriptFullName
            End Select
            'Get script path.
            mScriptPath = Left$(mScriptFullName, InStrRev(mScriptFullName, "\") - 1)
            'Get scripting Language.
            mLanguage = Mid$(mScriptFullName, InStrRev(mScriptFullName, ".") + 1)
            Select Case UCase$(mLanguage)
                Case "VBS", "VBE"
                    mLanguage = "VBScript"
                Case "JS", "JSE"
                    mLanguage = "JScript"
                Case Else
                    mOtherErrorInfo = mLanguage
                    ParseCommands = poeBadExtension
                    Exit Function
            End Select
            mScriptTimeout = -1 'Default.
            Set mArguments = New Collection 'Empty.
            If UBound(Parts) = 0 Then
                ParseCommands = poeNoError
                Exit Function
            End If
            'Get possible //T: switch.
            If UCase$(Left$(Parts(1), 4)) = "//T:" Then
                If Len(Parts(1)) < 5 Then
                    Usage
                    ParseCommands = poeShowedUsage
                    Exit Function
                End If
                On Error Resume Next
                I = CInt(Mid$(Parts(1), 5))
                If Err Then
                    Usage
                    ParseCommands = poeShowedUsage
                    Exit Function
                End If
                If I >= 0 Then
                    mScriptTimeout = 1000& * CLng(I)
                End If
                ScriptArgsBegin = 2
            Else
                ScriptArgsBegin = 1
            End If
            'Get mArguments Collection.
            For I = ScriptArgsBegin To UBound(Parts)
                mArguments.Add Parts(I)
            Next
            'Get mArgumentsLine String.
            If Left$(strCommand, 1) = """" Then
                ScriptArgsPos = InStr(2, strCommand, """")
            Else
                ScriptArgsPos = 1
            End If
            For I = 1 To ScriptArgsBegin
                ScriptArgsPos = InStr(ScriptArgsPos + 1, strCommand, " ")
                If ScriptArgsPos < 1 Then Exit For
            Next
            If ScriptArgsPos > 0 Then mArgumentsLine = Mid$(strCommand, ScriptArgsPos + 1)
            
            ParseCommands = poeNoError
    End Select
End Function

Private Sub Usage()
    MsgBox App.CompanyName & vbNewLine _
         & App.FileDescription & " Version " & CStr(App.Major) _
         & "." & CStr(App.Minor) & "." & CStr(App.Revision) & vbNewLine & vbNewLine _
         & "Usage: " & App.EXEName & " scriptname.extension [options...] [arguments...]" _
         & vbNewLine & vbNewLine _
         & "Options:" & vbNewLine _
         & "//T:nn" & vbTab & "Timeout in seconds: Maximum time script is permitted" _
         & vbNewLine _
         & vbTab & "to run.  The value 0 means ""system detects hang"" and" _
         & vbNewLine _
         & vbTab & "-1 (default) means ""never time out.""", _
           vbOKOnly Or vbInformation
End Sub

Private Sub Class_Initialize()
    Randomize
End Sub
